from getpass import getpass
from time import sleep
from abc import ABC, abstractmethod

import selenium.common.exceptions


class Login(ABC):
    def __init__(self, **kwargs):
        self.__login_data = {}
        self.__login_url = None
        try:
            self.driver = kwargs["driver"].driver
        except (AttributeError, KeyError): pass

    @abstractmethod
    def doLogin(self): raise NotImplementedError

    @property
    def login_data(self): return self.__login_data

    @login_data.setter
    def login_data(self, data):
        try:
            self.__login_data = {"user": data["user"], "passwd": data["passwd"]} 
        except KeyError:
            user = str(input("User: "))
            passwd = getpass("Password: ")
            self.__login_data = {"user": user, "passwd": passwd}

    @property
    def login_url(self): return self.__login_url

    @login_url.setter
    def login_url(self, platform):
        plat_login = {
            "google": "https://accounts.google.com/Login?hl=pt-BR",
            "meet": "https://accounts.google.com/Login?hl=pt-BR",
            "zoom": "https://zoom.us/google_oauth_signin"
        }
        try: self.__login_url = plat_login[platform]
        except KeyError:
            print("ERROR ****** ******** Platform not available! ******")


class GoogleLogin(Login):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def doLogin(self):
        try: self.driver.get(self.login_url)
        except AttributeError:
            print("ERROR ****** WEB DIVER OR LOGIN URL UNSET! ******")
            raise

        #USER
        try:
            self.driver.find_element_by_id("identifierId").send_keys(self.login_data["user"])
            self.driver.find_element_by_id("identifierNext").click()
            try:
                if self.driver.find_element_by_class_name("o6cuMc"):
                    print("ERROR ****** Login failed. Check user and try again! ******")
                    return
            except selenium.common.exceptions.NoSuchElementException: pass
        except (selenium.common.exceptions.NoSuchElementException, selenium.common.exceptions.ElementClickInterceptedException):
            print("ERROR ****** Login failed. No element found! Couldn't stablish connection ******")
            return
            
        #PASSWORD
        for _ in range(15):
            try:
                self.driver.find_element_by_name("password").send_keys(self.login_data["passwd"])
                break
            except (selenium.common.exceptions.NoSuchElementException, selenium.common.exceptions.ElementNotInteractableException):
                sleep(1)
        try:
            self.driver.find_element_by_id("passwordNext").click()
            try:
                if self.driver.find_element_by_class_name("EjBTad"):
                    print("ERROR ****** Login failed. Check your password and try again! ******")
                    return
            except selenium.common.exceptions.NoSuchElementException: pass
        except (selenium.common.exceptions.NoSuchElementException, selenium.common.exceptions.ElementClickInterceptedException):
                print("ERROR ****** Login failed. No element found! Couldn't stablish connection ******")
                return



from time import sleep
import selenium.common.exceptions

from .attend import AttendMeet

class GoogleMeet(AttendMeet):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def goto_meet(self):
        try: self.driver.get(self.meet_url)
        except selenium.common.exceptions.InvalidArgumentException:
            print("ERROR ****** Meeting code was not properly set. Please, provide a valid one and try again! ******")
            raise
        except selenium.common.exceptions.InvalidSessionIdException:
            print("ERROR ****** INVALID SESSION! ******")
            raise
        except AttributeError:
            print("ERROR ****** WEB DRIVER NOT FOUND! ******")
            raise

        for _ in range(25):
            try:
                self.driver.find_element_by_class_name("uArJ5e UQuaGc Y5sE8d uyXBBb xKiqt M9Bg4d".replace(" ", ".")).click()
                break
            except selenium.common.exceptions.NoSuchElementException:
                sleep(1)
                continue

        return

    def set_meeting_url(self, code):
        code_len = len(code)
        mCode = ""

        if "https://meet.google.com/" in code:
            self.meet_url = code
        elif "meet.google.com/" in code:
            self.meet_url= "{0}{1}".format("https://", code)
        else:
            if type(code) != type(0) and ((code_len == 12 and code[3] == "-" and code[8] == "-") or code_len == 10):
                for crc in code:
                    if type(crc) == type(0):
                        print("ERROR ****** Meeting code must not contain numbers! ******")
                    else:
                        mCode = code
            else: print("ERROR ****** Meeting code not accepted! Please check again ******")

            self.meet_url="https://meet.google.com/%s" % mCode
        return



import selenium.common.exceptions

from .googleMeet import GoogleMeet

class ZoomMeet(GoogleMeet):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def goto_meet(self):
        try:
            self.driver.get(self.meet_url)
            self.driver.find_elements_by_tag_name("a")[4].click()
        except (selenium.common.exceptions.InvalidArgumentException, selenium.common.exceptions.NoSuchElementException):
            print("ERROR ****** Meeting code was not properly set. Please, provide a valid one and try again! ******")
            raise
        except selenium.common.exceptions.InvalidSessionIdException:
            print("ERROR ****** INVALID SESSION! ******")
            raise
        except AttributeError:
            print("ERROR ****** WEB DRIVER NOT FOUND! ******")
            raise

        self.driver.find_element_by_id("joinBtn").click()
        return

    def set_meeting_url(self, code):
        if "https://zoom.us/j/" in code:
            self.meet_url = code
        else: self.meet_url="https://zoom.us/j/%s" % code



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="ethmeet",
    version="1.5.3-b.3",
    author="Lo Han",
    author_email="lohan.uchsa@protonmail.com",
    description="Online meeting automation tool",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/sourcerer0/ethmeet",
    packages=setuptools.find_packages(),
    keywords="bot firefox automation browser selenium meeting zoom google",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: POSIX :: Linux",
    ],
    python_requires='>=3.6',
    scripts=[
        'scripts/gecko_install',
        'scripts/QT_DEVICE',
    ],
    install_requires=[
        'selenium>=3.141.0',
        'urllib3>=1.26.3',
        'cryptography>=3.2'
    ]
)


from abc import ABC, abstractmethod

class AttendMeet(ABC):
    def __init__(self, **kwargs):
        self.meet_url = None
        try:
            self.driver = kwargs["driver"].driver
        except (AttributeError, KeyError): pass

        print(kwargs["code"])
        try: self.set_meeting_url(kwargs["code"])
        except KeyError: pass


    @abstractmethod
    def goto_meet(self): raise NotImplementedError

    @abstractmethod
    def set_meeting_url(self, code): raise NotImplementedError



from selenium.webdriver.firefox.webdriver import WebDriver

import os
EXECUTABLE_PATH = os.environ["HOME"] + "/geckodriver"

class Driver():
    def __init__(self):
        self.__driver = None

    @property
    def driver(self): return self.__driver

    @driver.setter
    def driver(self, browser):
        if browser == "firefox": self.__driver = WebDriver(executable_path=EXECUTABLE_PATH)
        else: self.__driver = None
